"""Module with functions for 'deploy' subpackage."""

from __future__ import annotations
import os

from typing_extensions import Literal

from mypythontools.paths import validate_path, PathLike
from mypythontools.misc import delete_files, print_progress
from mypythontools.system import check_script_is_available, terminal_do_command, PYTHON

from mypythontools_cicd.project_paths import PROJECT_PATHS
from mypythontools_cicd.venvs import Venv


def deploy_to_pypi(
    setup_path: None | PathLike = None,
    clean: bool = True,
    verbosity: Literal[0, 1, 2] = 1,
    pep517: bool = False,
    venv: None | Venv = Venv("venv"),
) -> None:
    """Publish python library to PyPi.

    Username and password are set with env vars `TWINE_USERNAME` and `TWINE_PASSWORD`.

    Note:
        You need working `setup.py` file. If you want to see example, try the one from project-starter on

        https://github.com/Malachov/mypythontools/blob/master/content/project-starter/setup.py

    Args:
        setup_path (None | PathLike, optional): Function suppose, that there is a setup.py somewhere in cwd.
            If not, path will be inferred. Build and dist folders will be created in same directory.
            Defaults to None.
        clean (bool, optional): Whether delete created build and dist folders.
        verbosity (Literal[0, 1, 2], optional): If 0, prints nothing, if 1, then one line description of what
            happened is printed. If 3, all the results from terminal are printed. Defaults to 1.
        pep517 (bool, optional): Whether using PEP 517, that use pyproject.toml to build distribution. Without
            it it's faster, but with it, pyproject.TOML can be used. Defaults to False.
        venv (None | Venv): Venv used for building distribution. Defaults to Venv("venv").
    """
    print_progress("Deploying to PyPi", verbosity > 0)
    verbose = verbosity == 2

    usr = os.environ.get("TWINE_USERNAME")
    password = os.environ.get("TWINE_PASSWORD")

    if not usr or not password:
        raise KeyError("Setup env vars TWINE_USERNAME and TWINE_PASSWORD to use deploy.")

    check_script_is_available("twine", "twine")

    setup_path = (
        validate_path(setup_path, "Deploy with `deploy_to_pypi` failed", "setup.py")
        if setup_path
        else PROJECT_PATHS.root / "setup.py"
    )

    setup_dir_path = setup_path.parent

    dist_path = setup_dir_path / "dist"
    build_path = setup_dir_path / "build"

    delete_files(dist_path)
    delete_files(build_path)

    activate_prefix_command = "" if not venv else venv.activate_prefix_command

    if pep517:
        build_command = f"{activate_prefix_command} {PYTHON} -m build --wheel --sdist"
    else:
        build_command = f"{activate_prefix_command} {PYTHON} setup.py sdist bdist_wheel"

    terminal_do_command(
        build_command,
        cwd=setup_dir_path.as_posix(),
        verbose=verbose,
        error_header=(
            "Build python distribution for PyPi deployment failed. Change of 'pep517' parameter may help."
        ),
    )

    command = "twine upload dist/*"

    terminal_do_command(
        command,
        cwd=setup_dir_path.as_posix(),
        verbose=verbose,
        error_header="Deploying to PyPi failed.",
    )

    if clean:
        delete_files(dist_path)
        delete_files(build_path)



"""Module with functions for 'packages' subpackage."""

from __future__ import annotations
from typing import Sequence, Any
from pathlib import Path
import os
import pkg_resources
import re

from typing_extensions import Literal
from mypythontools.paths import validate_path, PathLike

from mypythontools_cicd.project_paths import PROJECT_PATHS

# Lazy import
# from setuptools import find_packages


def get_requirements_files(
    requirements: Literal["infer"] | PathLike | Sequence[PathLike], path: PathLike = PROJECT_PATHS.root
) -> list[Path]:
    """Consolidate requirements into list of paths.

    Args:
        requirements (Literal["infer"] | PathLike | Sequence[PathLike]): E.g. ["requirements.txt",
            "requirements_dev.txt"]. If 'infer', then every file where requirements is in the name is used.
            It working with one level of nested folder if requirements is in the name of the file or folder.
        path (PathLike, optional): If using just names or relative path, and not found, define the root.
            It's also necessary when using another referenced files. If inferring files, it's used to search.
            Defaults to PROJECT_PATHS.root.

    Returns:
        list[Path]: List of paths to requirements files.

    Raises:
        RuntimeError: If no requirements found with 'infer'.
    """

    path = validate_path(path)

    if requirements == "infer":

        requirements_files = []
        for i in [*path.glob("*"), *path.glob("*/*")]:
            if "requirements" in i.as_posix().lower() and i.suffix == ".txt":
                requirements_files.append(i)

        if not requirements_files:
            raise RuntimeError("No requirements found.")

    else:
        if isinstance(requirements, (Path, str, os.PathLike)):
            requirements = [requirements]

        # Enforce Path type and validate if exists
        requirements_files = []
        for req in requirements:
            existing_file = None

            if Path(req).exists():
                existing_file = Path(req)
            elif (path / req).exists():
                existing_file = path / req
            else:
                for i in [*path.glob("*"), *path.glob("*/*")]:
                    if i.name == req:
                        existing_file = i
                        break

                if not existing_file:
                    raise FileNotFoundError(
                        f"Requirements file {req} not found. File may be referenced from another "
                        "requirements. Try to use appropriate 'path' parameter."
                    )
            requirements_files.append(existing_file)

    return requirements_files


def get_requirements(
    files: Literal["infer"] | PathLike | Sequence[PathLike], path: PathLike = PROJECT_PATHS.root
) -> list[str]:
    """Get requirements into variable usually used in setup.py.

    Args:
        files (Literal["infer"] | PathLike | Sequence[PathLike]): E.g. ["requirements.txt",
            "requirements_dev.txt"]. If 'infer', then every file where requirements is in the name is used.
            It can be also absolute paths.
        path (PathLike, optional): If using just names or relative path, and not found, define the root.
            It's also necessary when using another referenced files. If inferring files, it's used to search.
            Defaults to PROJECT_PATHS.root.

    Returns:
        list[str]: List of requirements

    Raises:
        RuntimeError: If no requirements files found with 'infer'.
    """
    files = get_requirements_files(files, path)
    all_requirements = []

    for file in files:

        with open(file) as f:
            requirements = f.readlines()
        requirements = [i.strip("\r\n") for i in requirements if i.strip("\r\n")]

        # narrow_requirements calls recursively get_requirements so it can have another -r reference
        requirements = narrow_requirements(requirements, path)
        requirements = [str(requirement) for requirement in pkg_resources.parse_requirements(requirements)]
        all_requirements.extend(requirements)

    return all_requirements


def narrow_requirements(requirements: list[str], path: PathLike = "requirements") -> list[str]:
    """If there is another requirements file reference in requirements.txt, this will replace it with its
    requirements it contains.

    Args:
        requirements (list[str]): List of requirements. You can parse it from file with `get_requirements`.
        path (PathLike, optional): Path with referenced files. Defaults to "requirements".

    Returns:
        list[str]: List of plain pep 508 compatible requirements.
    """
    path = validate_path(path, "'narrow_requirements' failed", "Path with referenced requirements")

    for i in requirements:
        if i.startswith("-r "):
            req_name = i[3:].strip("\r\n")
            requirements.extend(get_requirements(req_name, path=path))
    requirements = [i for i in requirements if not i.startswith("-r ")]
    return requirements


def validate_version(version: str):
    """Check whether parsed version is valid.

    Args:
        version (str): E.g "1.0.1"

    Returns:
        bool: Whether is valid.
    """
    return version == "increment" or (
        len(version.split(".")) == 3 and all([i.isdecimal() for i in version.split(".")])
    )


def get_readme() -> str:
    """Get README content into variable usually used in setup.py.

    Returns:
        str: Content.
    """
    with open(PROJECT_PATHS.readme.as_posix()) as readme_file:
        readme = readme_file.read()
    return readme


def get_version(init_path: None | PathLike = None) -> str:
    """Get version info from `__init__.py` file.

    Args:
        init_path (None | PathLike, optional): Path to `__init__.py` file. If None, will be inferred.
            Defaults to None.

    Returns:
        str: String of version from `__init__.py`.

    Raises:
        ValueError: If no `__version__` is find. Try set init_path...

    Example:
        >>> version = get_version()
        >>> len(version.split(".")) == 3 and all([i.isdecimal() for i in version.split(".")])
        True
    """
    init_path = init_path if init_path else PROJECT_PATHS.init
    try:
        init_path = validate_path(init_path)
    except FileNotFoundError:
        raise FileNotFoundError(
            f"Setting version failed. '__init__.py' not found on '{init_path}'."
        ) from None

    with open(init_path.as_posix(), "r") as init_file:
        version = re.findall('__version__ = "(.*)"', init_file.read())[0]

    if validate_version(version):
        return version
    else:
        raise RuntimeError("Version not found in __init__.py")


def set_version(
    version: str = "increment",
    init_path: None | PathLike = None,
) -> None:
    """Change your version in your `__init__.py` file.

    Args:
        version (str, optional): Form that is used in `__init__`, so for example "1.2.3". Do not use 'v'
            appendix. If version is 'increment', it will increment your `__version__` in you `__init__.py` by
            0.0.1. Defaults to "increment".
        init_path (None | PathLike, optional): Path of file where `__version__` is defined.
            Usually `__init__.py`. If None, will be inferred. Defaults to None.

    Raises:
        ValueError: If no `__version__` is find.
    """
    init_path = (
        validate_path(init_path, "Setting version failed", "__init__.py") if init_path else PROJECT_PATHS.init
    )

    is_valid = validate_version(version)

    if not is_valid:
        raise ValueError(
            "Version not validated. Version has to be of form '1.2.3'. Three digits and two dots. "
            f"You used {version}"
        )

    with open(init_path.as_posix(), "r") as init_file:

        list_of_lines = init_file.readlines()

        found = False

        for i, j in enumerate(list_of_lines):
            if j.startswith("__version__"):

                found = True

                delimiter = '"' if '"' in j else "'"
                delimited = j.split(delimiter)

                if version == "increment":
                    delimited[1] = increment_version(delimited[1])
                else:
                    delimited[1] = version

                list_of_lines[i] = delimiter.join(delimited)
                break

        if not found:
            raise ValueError("__version__ variable not found in __init__.py. Try set init.")

    with open(init_path.as_posix(), "w") as init_file:

        init_file.writelines(list_of_lines)


def increment_version(version: str) -> str:
    """Increment patch version by one.

    Args:
        version (str): String format. It can include 'v' prefix.

    Returns:
        str: Incremented version

    Example:
        >>> increment_version("1.2.3")
        '1.2.4'
        >>> increment_version("v1.2.3")
        'v1.2.4'
    """
    version_list = version.split(".")
    version_list[2] = str(int(version_list[2]) + 1)
    return ".".join(version_list)


def get_package_setup_args(
    name: str, development_status: Literal["alpha", "beta", "stable"] = "alpha"
) -> dict[str, Any]:
    """Get setup args, that usually repeats across projects.

    Args:
        name (str): Name used to generate some params.
        development_status (Literal['alpha', 'beta', 'stable']): Project phase. Defaults to 'alpha'.

    Returns:
        dict[str, Any]: Attributes used in `setup.py`. E.g. license, platform
            or long_description_content_type.
    """
    from setuptools import find_packages

    status = {
        "alpha": "Development Status :: 3 - Alpha",
        "beta": "Development Status :: 4 - Beta",
        "stable": "Development Status :: 5 - Production/Stable",
    }[development_status]

    classifiers = [
        status,
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]

    return {
        "license": "mit",
        "include_package_data": True,
        "long_description_content_type": "text/markdown",
        "packages": find_packages(exclude=("tests**",)),
        "platforms": "any",
        "python_requires": ">=3.7",
        "url": f"https://github.com/Malachov/{name}",
        "name": name,
        "version": get_version(),
        "classifiers": classifiers,
    }


personal_setup_args_preset = {
    "author_email": "malachovd@seznam.cz",
    "author": "Daniel Malachov",
}



"""Module with functions for 'cicd' subpackage."""

from __future__ import annotations
from typing import Sequence
import os
import sys

from typing_extensions import Literal

from mypythontools.config import Config, MyProperty
from mypythontools.misc import GLOBAL_VARS, EMOJIS, print_progress
from mypythontools.paths import PathLike

from .. import git
from .. import venvs
from .. import tests
from ..deploy import deploy_to_pypi
from ..docs import docs_regenerate
from ..misc import reformat_with_black
from ..project_paths import PROJECT_PATHS
from ..venvs import Venv

from .. import packages


class PipelineConfig(Config):
    """Allow to setup CICD pipeline."""

    def __init__(self) -> None:
        """Create subconfigs."""
        self.test: tests.TestConfig = tests.default_test_config

    @MyProperty
    def do_only(
        self,
    ) -> Literal[
        None,
        "prepare_venvs",
        "reformat",
        "test",
        "docs",
        "sync_requirements",
        "git_commit_all",
        "git_push",
        "deploy",
    ]:
        """Run just single function from pipeline, ignore the others.

        Type:
            Literal[
                None, "prepare_venvs", "reformat", "test", "docs", "sync_requirements", "git_commit_all",
                "git_push", "deploy"
            ]

        Default:
            None

        Reason for why to call it form here and not directly is to be able to use sys args or single command
        line entrypoint.
        """
        return None

    @MyProperty
    def venv(self) -> None | Venv:
        """Used venv. Now used only in "deploy" function for build.

        Type:
            None | Venv

        Default:
            Venv("venv")
        """
        return Venv("venv")

    @MyProperty
    def prepare_venvs(self) -> None | list[str]:
        """Create venvs with defined versions.

        Type:
            None | list[str]

        Default:
            None
        """
        return None

    @MyProperty
    def prepare_venvs_path(self) -> PathLike:
        """Prepare venvs in defined path.

        Type:
            str

        Default:
            "venv"
        """
        return "venv"

    @MyProperty
    def reformat(self) -> bool:
        """Reformat all python files with black. Setup parameters in pyproject.toml.

        Type:
            bool

        Default:
            True
        """
        return True

    @MyProperty
    def set_version(self) -> None | str:
        """Overwrite __version__ in __init__.py.

        Type:
            str

        Default:
            'increment'.

        Version has to be in format like '1.0.3' three digits and two dots. If 'None', nothing will happen. If
        'increment', than it will be updated by 0.0.1..
        """
        return "increment"

    @MyProperty
    def docs(self) -> bool:
        """Define whether generate sphinx apidoc and generate rst files for documentation.

        Type:
            bool

        Default:
            True

        Some files in docs source can be deleted - check `docs` docstrings for details.
        """
        return True

    @MyProperty
    def sync_requirements(self) -> None | Literal["infer"] | PathLike | Sequence[PathLike]:
        """Check requirements.txt and update all the libraries.

        Type:
            None | Literal["infer"] | PathLike | Sequence[PathLike]

        Default:
            None

        You can use path to requirements, list of paths. If "infer", auto detected (all requirements), not
        recursively, only on defined path.
        """
        return None

    @MyProperty
    def sync_requirements_path(self) -> PathLike:
        """Define the root if using just names or relative path, and not found.

        Type:
            PathLike

        Default:
            PROJECT_PATHS.root

        It's also necessary when using another referenced files. If inferring files, it's used to search.
        Defaults to PROJECT_PATHS.root.
        """
        return PROJECT_PATHS.root

    @MyProperty
    def git_commit_all(self) -> None | str:
        """Whether take all the changes in repository and create a commit with these changes.

        Note:
            !!! Be cautious here if using with `git_push` !!!

        Type:
            None | str

        Default:
            'New commit'
        """
        return "New commit"

    @MyProperty
    def git_push(self) -> bool:
        """Whether push to repository.

        Type:
            bool

        Default:
            True
        """
        return True

    @MyProperty
    def tag(self) -> str | None:
        """Tag. E.g 'v1.1.2'. If '__version__', get the version.

        Type:
            str | None

        Default:
            '__version__'
        """
        return "__version__"

    @MyProperty
    def tag_message(self) -> str:
        """Tag message.

        Type:
            str

        Default:
            'New version'
        """
        return "New version"

    @MyProperty
    def deploy(self) -> bool:
        """Deploy to PYPI.

        `TWINE_USERNAME` and `TWINE_PASSWORD` are used for authorization.

        Type:
            bool

        Default:
            False
        """
        return False

    @MyProperty
    def allowed_branches(self) -> None | Sequence[str]:
        """Pipeline runs only on defined branches.

        Type:
            None | Sequence[str]

        Default:
            ["master", "main"]
        """
        return ["master", "main"]

    @MyProperty
    def verbosity(self) -> Literal[0, 1, 2]:
        """Pipeline runs only on defined branches.

        Type:
            Literal[0, 1, 2]

        Default:
            1
        """
        return 1


default_pipeline_config = PipelineConfig()


def cicd_pipeline(
    config: PipelineConfig = default_pipeline_config,
) -> None:
    """Run pipeline for pushing and deploying an app or a package.

    Can run tests, generate rst files for sphinx docs, push to github and deploy to pypi. All params can be
    configured not only with function params, but also from command line with params and therefore callable
    from terminal and optimal to run from IDE (for example with creating simple VS Code task).

    Some function suppose some project structure (where are the docs, where is `__init__.py` etc.).
    If you are issuing some error, try functions directly, find necessary paths in parameters
    and set paths that are necessary in paths module.

    Note:
        Beware, that by default, it creates a commit and add all the changes, not only the staged ones.

    When using sys args for boolean values, always define True or False.

    There is command line entrypoint called `mypythontools_cicd`. After mypythontools is installed, you can
    use it in terminal like::

        mypythontools_cicd --do_only reformat

    Args:
        config (PipelineConfig, optional): PipelineConfig object with CICD pipeline configuration. Just import
            default_pipeline_config and use intellisense and help tooltip with description. It is also
            possible to configure all the params with CLI args from terminal.
            Defaults to `default_pipeline_config`.

    Example:
        Recommended use is from IDE (for example with Tasks in VS Code). Check utils docs for how to use it.
        You can also use it from python... ::

            from mypythontools_cicd.cicd import cicd_pipeline, default_pipeline_config

            default_pipeline_config.deploy = True
            # default_pipeline_config.do_only = ""


            if __name__ == "__main__":
                # All the parameters can be overwritten via CLI args
                cicd_pipeline(config=default_pipeline_config)

        It's also possible to use CLI and configure it via args. This example just push repo to PyPi. ::

            python path-to-project/utils/push_script.py --do_only deploy

    Another way how to run it is to use IDE. For example in VS Code you can use Tasks. Check `cicd`
    docs for examples.
    """
    if not GLOBAL_VARS.is_tested:
        config.do.with_argparse()

    do_only = config.do_only

    # If do_only change subconfig value and not config, change it
    if do_only == "test":
        do_only = "run_tests"

    # For do_only True value must be set for boolean even if not configured

    if do_only:
        # There are some boolean variables where is enough to tell do_only name
        do_only_booleans = ["reformat", "run_tests", "git_push", "deploy"]
        do_only_value = config[do_only]
        if do_only in do_only_booleans:
            do_only_value = True
        else:
            # There are some variables, where not only variable is chosen, but also a value must be set
            if not do_only_value:
                raise RuntimeError(
                    "If you are doing just one step from `cicd_pipeline` with 'do_only', for non boolean you "
                    "must have also configured a value in the config."
                )

        config.do.update(
            {
                "prepare_venvs": None,
                "reformat": False,
                "run_tests": False,
                "docs": False,
                "sync_requirements": None,
                "git_commit_all": None,
                "git_push": False,
                "deploy": False,
                "set_version": None,
                "tag": None,
            }
        )
        config.do.update({do_only: do_only_value})

    verbosity = config.verbosity
    progress_is_printed = verbosity > 0

    if config.allowed_branches:
        git.check_branch(config.allowed_branches)

    # Do some checks before run pipeline so not need to rollback eventually
    if config.deploy:
        usr = os.environ.get("TWINE_USERNAME")
        pas = os.environ.get("TWINE_PASSWORD")

        if not usr or not pas:
            raise KeyError("Setup env vars TWINE_USERNAME and TWINE_PASSWORD to use deploy.")

    if config.tag:
        if config.tag == "__version__":
            tag = config.set_version
            if tag == "increment":
                tag = f"v{packages.increment_version(packages.get_version())}"
            if not tag:
                tag = packages.get_version()
        else:
            tag = config.tag
        git.check_tag(tag)

    if config.prepare_venvs:
        venvs.prepare_venvs(
            path=config.prepare_venvs_path, versions=config.prepare_venvs, verbosity=verbosity
        )

    if config.sync_requirements:
        if not venvs.is_venv:
            raise RuntimeError("'sync_requirements' available only if using virtualenv.")
        my_venv = venvs.Venv(sys.prefix)
        my_venv.sync_requirements(
            config.sync_requirements, verbosity=verbosity, path=config.sync_requirements_path
        )

    if config.test:
        tests.run_tests(config.test)

    if config.reformat:
        reformat_with_black()

    if config.set_version:
        print_progress("Setting version", progress_is_printed)
        original_version = packages.get_version()
        packages.set_version(config.set_version)

    if config.docs:
        docs_regenerate(verbosity=verbosity)

    if config.git_commit_all:
        git.commit_all(config.git_commit_all, verbosity=verbosity)

    try:
        if config.git_push:
            git.push(
                tag=config.tag,
                tag_message=config.tag_message,
                verbosity=verbosity,
            )

    except Exception as err:  # pylint: disable=broad-except
        if config.set_version:
            packages.set_version(original_version)  # type: ignore

        raise RuntimeError(
            f"{3 * EMOJIS.DISAPPOINTMENT} Utils pipeline failed {3 * EMOJIS.DISAPPOINTMENT} \n\n"
            f"{'Original version restored. ' if config.set_version else ''}Nothing was pushed to repo, "
            "you can restart pipeline. "
            f"{'All changes already committed.' if config.git_commit_all else ''}"
        ) from err

    try:
        if config.deploy:
            deploy_to_pypi(verbosity=verbosity, venv=config.venv)

    except Exception as err:  # pylint: disable=broad-except
        raise RuntimeError(
            f"{3 * EMOJIS.DISAPPOINTMENT} Deploy failed {3 * EMOJIS.DISAPPOINTMENT} \n\n"
            "Already pushed to repository. Deploy manually."
            f"{'All changes already committed.' if config.git_commit_all else ''}"
            f"{'Version already changed.' if config.set_version else ''}"
        ) from err

    print_progress(f"{3 * EMOJIS.PARTY} Finished {3 * EMOJIS.PARTY}", True)



"""Module with functionality around Continuous Integration and Continuous Delivery.

.. image:: https://img.shields.io/pypi/pyversions/mypythontools_cicd.svg
    :target: https://pypi.python.org/pypi/mypythontools_cicd/
    :alt: Python versions

.. image:: https://badge.fury.io/py/mypythontools_cicd.svg
    :target: https://badge.fury.io/py/mypythontools_cicd
    :alt: PyPI version

.. image:: https://pepy.tech/badge/mypythontools_cicd
    :target: https://pepy.tech/project/mypythontools_cicd
    :alt: Downloads

.. image:: https://img.shields.io/lgtm/grade/python/github/Malachov/mypythontools_cicd.svg
    :target: https://lgtm.com/projects/g/Malachov/mypythontools_cicd/context:python
    :alt: Language grade: Python

.. image:: https://readthedocs.org/projects/mypythontools_cicd/badge/?version=latest
    :target: https://mypythontools_cicd.readthedocs.io/en/latest/?badge=latest
    :alt: Documentation Status

.. image:: https://img.shields.io/badge/License-MIT-yellow.svg
    :target: https://opensource.org/licenses/MIT
    :alt: License: MIT

.. image:: https://codecov.io/gh/Malachov/mypythontools_cicd/branch/master/graph/badge.svg
    :target: https://codecov.io/gh/Malachov/mypythontools_cicd
    :alt: Codecov

Why to use this and not Travis or Circle CI? It's local and it's fast. You can setup it as a task in IDE and
if some phase fails, you know it soon and before pushing to repo.

You can also import mypythontools in your CI/CD and use it there of course.

If you are not sure whether the structure of your app will work with this code, check
`project-starter-cookiecutter` on https://github.com/Malachov/project-starter-cookiecutter

Links
-----

Official documentation - https://mypythontools_cicd.readthedocs.io

Official repo - https://github.com/Malachov/mypythontools_cicd


Installation
------------

Python >=3.6 (Python 2 is not supported).

Install with::

    pip install mypythontools_cicd
    
There are many extras requirements that can be used with square brackets like for example

    pip install mypythontools_cicd[tests]

You can use `dev` which install libraries used during development like for example pylint. Most subpackages
has own extras. `'build','deploy', 'docs', 'git', 'misc', 'tests', 'venvs'` in particular.

There is also `cicd` which install all libraries necessary for cicd, but without dev dependencies. Last
extras is `all` which use dev as well as cicd.

Subpackages
-----------
Package is divided into several subpackages

:py:mod:`mypythontools_cicd.build_app`
--------------------------------------
Build your application to .exe with pyinstaller. It also builds javascript frontend with npm build if configured,
which is used mostly in PyVueEel applications.

:py:mod:`mypythontools_cicd.cicd`
----------------------------------
Pipeline for all the other submodules, that provide configurable CI/CD.

:py:mod:`mypythontools_cicd.deploy`
-----------------------------------
Build package and push it to PyPi.

:py:mod:`mypythontools_cicd.docs`
-----------------------------------
Provide documentation with sphinx.

:py:mod:`mypythontools_cicd.git`
--------------------------------
Works with git. You can check branch here, commit all changes or push to repository.

:py:mod:`mypythontools_cicd.misc`
--------------------------------
Miscellaneous functions that are too small to have own subpackage, like for example formatting with black.

:py:mod:`mypythontools_cicd.packages`
-------------------------------------
For example, you can work with requirements here. Usually used in 'setup.py'.

:py:mod:`mypythontools_cicd.project_paths`
------------------------------------------
Subpackage where you can get paths used in your project (path to README,  `__init__.py` etc.).

:py:mod:`mypythontools_cicd.tests`
----------------------------------
Runs tests in more venvs with different python versions, also with wsl linux if configured and create coverage.

:py:mod:`mypythontools_cicd.venvs`
----------------------------------
Works with virtual environments.


Mypythontools
=============

There is extra library in separate repository which is not about CICD, but normal python helpers.

https://github.com/Malachov/mypythontools

This can help you with a lot of stuff around CICD like getting project paths, generating docs, testing,
deploying to PyPi etc.
"""
from mypythontools_cicd import build_app, deploy, packages, cicd, project_paths, tests, venvs

__all__ = ["build_app", "deploy", "packages", "project_paths", "cicd", "tests", "venvs"]

__version__ = "0.0.35"

__author__ = "Daniel Malachov"
__license__ = "MIT"
__email__ = "malachovd@seznam.cz"



"""Helpers with git and github functionality."""

from mypythontools_cicd.git.git_internal import (
    commit_all,
    push,
    check_branch,
    check_tag,
    TagAlreadyExists,
    clone_locally,
)

__all__ = ["commit_all", "push", "check_branch", "check_tag", "TagAlreadyExists", "clone_locally"]
